﻿using Windows.Foundation;
using Windows.Foundation.Metadata;
using Windows.Graphics.Display;
using Windows.UI.Composition;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 작성자
        /// </summary>
        private Compositor compositor = Window.Current.Compositor;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "ExpressionAnimation 클래스 사용하기";

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 로드시 처리하기 - grid_Loaded(sender, e)

        /// <summary>
        /// 그리드 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void grid_Loaded(object sender, RoutedEventArgs e)
        {
            if(!ApiInformation.IsApiContractPresent("Windows.Foundation.UniversalApiContract", 8))
            {
                return;
            }

            string radius     = "(grid.ActualSize.X / 2)";
            string theta      = ".02 * " + radius + " + ((2 * Pi) / total) * index";
            string xOffset    = radius;
            string yOffset    = "0";
            string expression = string.Format("Vector3({0} * cos({1}) + {2}, {0} * sin({1}) + {3}, 0)", radius, theta, xOffset, yOffset);

            int elementCount = 8;

            for(int i = 0; i < elementCount; i++)
            {
                Button button = new Button() { Content = "버튼" };

                this.grid.Children.Add(button);

                ExpressionAnimation animation = this.compositor.CreateExpressionAnimation();

                animation.Expression = expression;

                animation.SetScalarParameter("index", i + 1);
                animation.SetScalarParameter("total", elementCount);

                animation.Target = "Translation";

                animation.SetExpressionReferenceParameter("grid", grid);

                button.StartAnimation(animation);
            }
        }

        #endregion
        #region 슬라이더 값 변경시 처리하기 - slider_ValueChanged(sender, e)

        /// <summary>
        /// 슬라이더 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void slider_ValueChanged(object sender, RangeBaseValueChangedEventArgs e)
        {
            if(this.grid == null)
            {
                return;
            }

            this.grid.Width = this.grid.Height = e.NewValue;
        }

        #endregion
    }
}